#include <QApplication>
#include <QLabel>
#include <QPushButton>
#include <QTimer>
#include <QtPlatformHeaders/QWindowsWindowFunctions>
#include <QVBoxLayout>
#include <QWidget>
#include <QDebug>

bool optBorderInFullScreen = true;

class Window : public QWidget
{
public:
    Window()
    {
        const auto label = new QLabel(this);
        const auto toggle_fullscreen_button = new QPushButton(QLatin1String("Toggle fullscreen"), this);
        const auto close_button = new QPushButton(QLatin1String("Close window"), this);

        const auto layout = new QVBoxLayout(this);
        layout->addWidget(label);
        layout->addWidget(toggle_fullscreen_button);
        layout->addWidget(close_button);

        setLayout(layout);

        const auto timer = new QTimer(this);
        timer->setInterval(100);
        timer->start();
        connect(timer, &QTimer::timeout, this, [this, label] {
            QString t;
            Qt::WindowStates st  = windowState();
            QDebug(&t) << "Window state: " << int(st) << st;
            qDebug().noquote() << t;
            label->setText(t);
        });

        connect(toggle_fullscreen_button, &QPushButton::clicked, this, [this] {
            setWindowState(windowState() ^ Qt::WindowFullScreen);
        });

        connect(close_button, &QPushButton::clicked, this, &QWidget::close);
    }

    bool event(QEvent *e) override
    {
        if (e->type() == QEvent::WinIdChange && optBorderInFullScreen) {
            if (const auto win_handle = windowHandle()) {
                qDebug() << "setHasBorderInFullScreen";
                QWindowsWindowFunctions::setHasBorderInFullScreen(win_handle, true);
            }
        }
        return QWidget::event(e);
    }
};

int main(int argc, char **argv)
{
    QApplication app(argc, argv);
    QStringList args  = app.arguments();
    args.pop_front();

    QString t = QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') +  app.platformName();
    qDebug() << t << args;

    if (args.contains("-b"))
        optBorderInFullScreen = false;

    Window w;
    w.setWindowTitle(t);
    w.setWindowState(Qt::WindowFullScreen);
    w.showMinimized();

    return app.exec();
}
